"""(Very) Simple Implementation of Artnet.

Python Version: 3.6
Source: http://artisticlicence.com/WebSiteMaster/User%20Guides/art-net.pdf


NOTES
- For simplicity: NET and SUBNET not used by default but optional

"""

import socket
import _thread
from stupidArtnet.ArtnetUtils import make_address_mask


class StupidArtnetServer():
    """(Very) simple implementation of an Artnet Server."""

    socket_server = None
    ARTDMX_HEADER = b'Art-Net\x00\x00P\x00\x0e'
    listeners = []

    def __init__(self, port=6454):
        """Initializes Art-Net server."""
        self.port = port  # Use provided port or default
        # By default, the server uses port 6454, no need to specify it.
        # If you need to change the Art-Net port, ensure the port is within the valid range for UDP ports (1024-65535).
        # Be sure that no other application is using the selected port on your network.
        
        # server active flag
        self.listen = True

        self.server_thread = _thread.start_new_thread(self.__init_socket, ())

    def __init_socket(self):
        """Initializes server socket."""
        # Bind to UDP on the correct PORT
        self.socket_server = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.socket_server.setsockopt(
            socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.socket_server.bind(('', self.port))  # Listen on any valid IP

        while self.listen:

            data, unused_address = self.socket_server.recvfrom(1024)

            # only dealing with Art-Net DMX
            if self.validate_header(data):

                # check if this address is in any registered listener
                for listener in self.listeners:

                    # is it the address we are listening to
                    if listener['address_mask'] == data[14:16]:

                        # check if the packet we've received is old
                        new_seq = data[12]
                        old_seq = listener['sequence']
                        # if there's a >50% packet loss it's not our problem
                        if new_seq == 0x00 or new_seq > old_seq or old_seq - new_seq > 0x80:
                            listener['sequence'] = new_seq

                            listener['buffer'] = list(data)[18:]

                            # check for registered callbacks
                            callback = listener['callback']
                            if callback is not None:
                                # choose the correct callback call based
                                # on the number of the function's parameters
                                try:
                                    from inspect import signature
                                    params = signature(callback).parameters
                                    params_len = len(params)
                                except ImportError:
                                    params_len = 2
                                
                                if params_len == 1:
                                    callback(listener['buffer'])
                                elif params_len == 2:
                                    addr_mask = listener['address_mask']
                                    addr = int.from_bytes(addr_mask, 'little')
                                    callback(listener['buffer'], addr)

    def __del__(self):
        """Graceful shutdown."""
        self.listeners.clear()
        self.close()

    def __str__(self):
        """Printable object state."""
        state = "===================================\n"
        state += "Stupid Artnet Listening\n"
        return state

    def register_listener(self, universe=0, sub=0, net=0,
                          is_simplified=True, callback_function=None):
        """Adds a listener to an Art-Net Universe.

        Args:
        universe - Universe to listen
        sub - Subnet to listen
        net - Net to listen
        is_simplified - Whether to use nets and subnet or universe only,
        see User Guide page 5 (Universe Addressing)
        callback_function - Function to call when new packet is received

        Returns:
        id - id of listener, used to delete listener if required
        """
        listener_id = len(self.listeners)
        new_listener = {
            'id': listener_id,
            'simplified': is_simplified,
            'address_mask': make_address_mask(universe, sub, net, is_simplified),
            'callback': callback_function,
            'buffer': [],
            'sequence': 0
        }

        self.listeners.append(new_listener)

        return listener_id

    def delete_listener(self, listener_id):
        """Deletes a registered listener.

        Args:
        listener_id - Id of listener to delete

        Returns:
        None
        """
        self.listeners = [
            i for i in self.listeners if not i['id'] == listener_id]

    def delete_all_listener(self):
        """Deletes all registered listeners.

        Returns:
        None
        """
        self.listeners = []

    def see_buffer(self, listener_id):
        """Show buffer values."""
        for listener in self.listeners:
            if listener.get('id') == listener_id:
                return listener.get('buffer')

        return "Listener not found"

    def get_buffer(self, listener_id):
        """Return buffer values."""
        for listener in self.listeners:
            if listener.get('id') == listener_id:
                return listener.get('buffer')
        print("Buffer object not found")
        return []

    def clear_buffer(self, listener_id):
        """Clear buffer in listener."""
        for listener in self.listeners:
            if listener.get('id') == listener_id:
                listener['buffer'] = []

    def set_callback(self, listener_id, callback_function):
        """Add / change callback to a given listener."""
        for listener in self.listeners:
            if listener.get('id') == listener_id:
                listener['callback'] = callback_function

    def set_address_filter(self, listener_id, universe, sub=0, net=0,
                           is_simplified=True):
        """Add / change filter to existing listener."""
        # make mask bytes
        address_mask = make_address_mask(
            universe, sub, net, is_simplified)

        # find listener
        for listener in self.listeners:
            if listener.get('id') == listener_id:
                listener['simplified'] = is_simplified
                listener['address_mask'] = address_mask
                listener['buffer'] = []

    def close(self):
        """Close UDP socket."""
        self.listen = False         # Set flag, so thread will exit

    @staticmethod
    def validate_header(header):
        """Validates packet header as Art-Net packet.

        - The packet header spells Art-Net
        - The definition is for DMX Artnet (OPCode 0x50)
        - The protocol version is 15

        Args:
        header - Packet header as bytearray

        Returns:
        boolean - comparison value

        """
        return header[:12] == StupidArtnetServer.ARTDMX_HEADER


def test_callback(data):
    """Test function, receives data from server callback."""
    print('Received new data \n', data)


if __name__ == '__main__':

    import time

    print("===================================")
    print("Namespace run")

    # Art-Net 4 definition specifies nets and subnets
    # Please see README and Art-Net user guide for details
    # Here we use the simplified default
    UNIVERSE_TO_LISTEN = 1

    # Initilize server, this starts a server in the Art-Net port
    a = StupidArtnetServer()

    # For every universe we would like to receive,
    # add a new listener with a optional callback
    # the return is an id for the listener
    u1_listener = a.register_listener(
        UNIVERSE_TO_LISTEN, callback_function=test_callback)

    # print object state
    print(a)

    # giving it some time for the demo
    time.sleep(3)

    # use the listener address to get data without a callback
    buffer = a.get_buffer(u1_listener)

    # Cleanup when you are done
    del a



import time
import socket
import unittest

from stupidArtnet import StupidArtnetServer


class Test(unittest.TestCase):
    """Test class for Artnet server."""

    artnet_header = b'Art-Net\x00\x00P\x00\x0e\x00\x00\x00\x00\x00\x08'
    dmx_packet = [1, 2, 3, 4, 5, 6, 7, 8]
    dmx_packet_bytes = b'\x01\x02\x03\x04\x05\x06\x07\x08'

    def setUp(self):
        """Creates UDP Client."""
        # Create a dummy UDP Client
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        # Instanciate StupidArtnetServer
        self.stupid = StupidArtnetServer()

        print(self.stupid)

        # register listener on universe 0
        self.listener = self.stupid.register_listener()

        # make an artnet packet on universe 0
        artnet_data = bytearray()
        artnet_data.extend(self.artnet_header)
        artnet_data.extend(self.dmx_packet_bytes)

        time.sleep(0.5)

        # send artnet data
        self.sock.sendto(artnet_data, ('localhost', 6454))

        time.sleep(0.2)

        # send random stuff
        self.sock.sendto(b'Hello world', ('localhost', 6454))

    def tearDown(self):
        """Destroy Objects."""
        # destroy UDP Server
        self.sock.close()

        # destroy artnet instance
        del self.stupid

    def test_buffer(self):
        """Assert that server received data and filtered correctly."""
        buffer = self.stupid.get_buffer(self.listener)

        # Test with a artnet header
        self.assertEqual(buffer, self.dmx_packet)

    def test_header(self):
        """Assert Art-Net header."""
        artdmx = b'Art-Net\x00\x00P\x00\x0e'
        typo = b'Art-Net\x00\x00\x00\x0e'

        # Test with a artnet header
        self.assertTrue(StupidArtnetServer.validate_header(artdmx))

        # Test header with typo on OP code
        self.assertFalse(StupidArtnetServer.validate_header(typo))


if __name__ == '__main__':
    unittest.main()



"""(Very) Simple Implementation of Artnet.

Python Version: 3.6
Source: http://artisticlicence.com/WebSiteMaster/User%20Guides/art-net.pdf

NOTES
- For simplicity: NET and SUBNET not used by default but optional

"""

import socket
import _thread
from time import sleep
from stupidArtnet.ArtnetUtils import shift_this, put_in_range


class StupidArtnet():
    """(Very) simple implementation of Artnet."""

    def __init__(self, target_ip='127.0.0.1', universe=0, packet_size=512, fps=30,
                 even_packet_size=True, broadcast=False, source_address=None, artsync=False, port=6454):
        """Initializes Art-Net Client.

        Args:
        targetIP - IP of receiving device
        universe - universe to listen
        packet_size - amount of channels to transmit
        fps - transmition rate
        even_packet_size - Some receivers enforce even packets
        broadcast - whether to broadcast in local sub
        artsync - if we want to synchronize buffer
        port - UDP port used to send Art-Net packets (default: 6454)

        Returns:
        None

        """
        # Instance variables
        self.target_ip = target_ip
        self.sequence = 0
        self.physical = 0
        self.universe = universe
        self.subnet = 0
        self.if_sync = artsync
        self.net = 0
        self.packet_size = put_in_range(packet_size, 2, 512, even_packet_size)
        self.packet_header = bytearray()
        self.buffer = bytearray(self.packet_size)
        self.port = port  
        # Use provided port or default 6454
        # By default, the server uses port 6454, no need to specify it.
        # If you need to change the Art-Net port, ensure the port is within the valid range for UDP ports (1024-65535).
        # Be sure that no other application is using the selected port on your network.

        self.make_even = even_packet_size

        self.is_simplified = True		# simplify use of universe, net and subnet

        # UDP SOCKET
        self.socket_client = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        if broadcast:
            self.socket_client.setsockopt(
                socket.SOL_SOCKET, socket.SO_BROADCAST, 1)

        # Allow speciying the origin interface
        if source_address:
            self.socket_client.setsockopt(
                socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            self.socket_client.bind(source_address)

        # Timer
        self.fps = fps
        self.__clock = None

        self.make_artdmx_header()
        
        if self.if_sync:
            self.artsync_header = bytearray()
            self.make_artsync_header()


    def __del__(self):
        """Graceful shutdown."""
        self.stop()
        self.close()


    def __str__(self):
        """Printable object state."""
        state = "===================================\n"
        state += "Stupid Artnet initialized\n"
        state += f"Target IP: {self.target_ip} : {self.port} \n"
        state += f"Universe: {self.universe} \n"
        if not self.is_simplified:
            state += f"Subnet: {self.subnet} \n"
            state += f"Net: {self.net} \n"
        state += f"Packet Size: {self.packet_size} \n"
        state += "==================================="

        return state


    def make_artdmx_header(self):
        """Make packet header."""
        # 0 - id (7 x bytes + Null)
        self.packet_header = bytearray()
        self.packet_header.extend(bytearray('Art-Net', 'utf8'))
        self.packet_header.append(0x0)
        # 8 - opcode (2 x 8 low byte first)
        self.packet_header.append(0x00)
        self.packet_header.append(0x50)  # ArtDmx data packet
        # 10 - prototocol version (2 x 8 high byte first)
        self.packet_header.append(0x0)
        self.packet_header.append(14)
        # 12 - sequence (int 8), NULL for not implemented
        self.packet_header.append(self.sequence)
        # 13 - physical port (int 8)
        self.packet_header.append(0x00)
        # 14 - universe, (2 x 8 low byte first)
        if self.is_simplified:
            # not quite correct but good enough for most cases:
            # the whole net subnet is simplified
            # by transforming a single uint16 into its 8 bit parts
            # you will most likely not see any differences in small networks
            msb, lsb = shift_this(self.universe)   # convert to MSB / LSB
            self.packet_header.append(lsb)
            self.packet_header.append(msb)
        # 14 - universe, subnet (2 x 4 bits each)
        # 15 - net (7 bit value)
        else:
            # as specified in Artnet 4 (remember to set the value manually after):
            # Bit 3  - 0 = Universe (1-16)
            # Bit 7  - 4 = Subnet (1-16)
            # Bit 14 - 8 = Net (1-128)
            # Bit 15     = 0
            # this means 16 * 16 * 128 = 32768 universes per port
            # a subnet is a group of 16 Universes
            # 16 subnets will make a net, there are 128 of them
            self.packet_header.append(self.subnet << 4 | self.universe)
            self.packet_header.append(self.net & 0xFF)
        # 16 - packet size (2 x 8 high byte first)
        msb, lsb = shift_this(self.packet_size)		# convert to MSB / LSB
        self.packet_header.append(msb)
        self.packet_header.append(lsb)


    def make_artsync_header(self):
        """Make ArtSync header"""
        self.artsync_header = bytearray()  # Initialize as empty bytearray
        # ID: Array of 8 characters, the final character is a null termination.
        self.artsync_header.extend(bytearray('Art-Net', 'utf8'))
        self.artsync_header.append(0x0)
        # OpCode: Defines the class of data within this UDP packet. Transmitted low byte first.
        self.artsync_header.append(0x00)
        self.artsync_header.append(0x52)
        # ProtVerHi and ProtVerLo: Art-Net protocol revision number. Current value =14.
        # Controllers should ignore communication with nodes using a protocol version lower than =14.
        self.artsync_header.append(0x0)
        self.artsync_header.append(14)
        # Aux1 and Aux2: Should be transmitted as zero.
        self.artsync_header.append(0x0)
        self.artsync_header.append(0x0)


    def send_artsync(self):
        """Send Artsync"""
        self.make_artsync_header()
        try:
            self.socket_client.sendto(self.artsync_header, (self.target_ip, self.port))
        except socket.error as error:
            print(f"ERROR: Socket error with exception: {error}")


    def show(self):
        """Finally send data."""
        packet = bytearray()
        packet.extend(self.packet_header)
        packet.extend(self.buffer)
        try:
            self.socket_client.sendto(packet, (self.target_ip, self.port))
            if self.if_sync:  # if we want to send artsync
                self.send_artsync()
        except socket.error as error:
            print(f"ERROR: Socket error with exception: {error}")
        finally:
            self.sequence = (self.sequence + 1) % 256


    def close(self):
        """Close UDP socket."""
        self.socket_client.close()

    # THREADING #

    def start(self):
        """Starts thread clock."""
        self.show()
        if not hasattr(self, "running"):
            self.running = True
        if self.running:
            sleep((1000.0 / self.fps) / 1000.0)
            _thread.start_new_thread(self.start, ())


    def stop(self):
        """Set flag so thread will exit."""
        self.running = False

    # SETTERS - HEADER #

    def set_universe(self, universe):
        """Setter for universe (0 - 15 / 256).

        Mind if protocol has been simplified
        """
        # This is ugly, trying to keep interface easy
        # With simplified mode the universe will be split into two
        # values, (uni and sub) which is correct anyway. Net will always be 0
        if self.is_simplified:
            self.universe = put_in_range(universe, 0, 255, False)
        else:
            self.universe = put_in_range(universe, 0, 15, False)
        self.make_artdmx_header()


    def set_subnet(self, sub):
        """Setter for subnet address (0 - 15).

        Set simplify to false to use
        """
        self.subnet = put_in_range(sub, 0, 15, False)
        self.make_artdmx_header()


    def set_net(self, net):
        """Setter for net address (0 - 127).

        Set simplify to false to use
        """
        self.net = put_in_range(net, 0, 127, False)
        self.make_artdmx_header()


    def set_packet_size(self, packet_size):
        """Setter for packet size (2 - 512, even only)."""
        self.packet_size = put_in_range(packet_size, 2, 512, self.make_even)
        self.make_artdmx_header()

    # SETTERS - DATA #

    def clear(self):
        """Clear DMX buffer."""
        self.buffer = bytearray(self.packet_size)


    def set(self, value):
        """Set buffer."""
        if len(value) != self.packet_size:
            print("ERROR: packet does not match declared packet size")
            return
        self.buffer = value


    def set_16bit(self, address, value, high_first=False):
        """Set single 16bit value in DMX buffer."""
        if address > self.packet_size:
            print("ERROR: Address given greater than defined packet size")
            return
        if address < 1 or address > 512 - 1:
            print("ERROR: Address out of range")
            return
        value = put_in_range(value, 0, 65535, False)

        # Check for endianess
        if high_first:
            self.buffer[address - 1] = (value >> 8) & 0xFF  # high
            self.buffer[address] = (value) & 0xFF 			# low
        else:
            self.buffer[address - 1] = (value) & 0xFF				# low
            self.buffer[address] = (value >> 8) & 0xFF  # high


    def set_single_value(self, address, value):
        """Set single value in DMX buffer."""
        if address > self.packet_size:
            print("ERROR: Address given greater than defined packet size")
            return
        if address < 1 or address > 512:
            print("ERROR: Address out of range")
            return
        self.buffer[address - 1] = put_in_range(value, 0, 255, False)


    def set_single_rem(self, address, value):
        """Set single value while blacking out others."""
        if address > self.packet_size:
            print("ERROR: Address given greater than defined packet size")
            return
        if address < 1 or address > 512:
            print("ERROR: Address out of range")
            return
        self.clear()
        self.buffer[address - 1] = put_in_range(value, 0, 255, False)


    def set_rgb(self, address, red, green, blue):
        """Set RGB from start address."""
        if address > self.packet_size:
            print("ERROR: Address given greater than defined packet size")
            return
        if address < 1 or address > 510:
            print("ERROR: Address out of range")
            return

        self.buffer[address - 1] = put_in_range(red, 0, 255, False)
        self.buffer[address] = put_in_range(green, 0, 255, False)
        self.buffer[address + 1] = put_in_range(blue, 0, 255, False)

    # AUX Function #

    def send(self, packet):
        """Set buffer and send straightaway.

        Args:
        array - integer array to send
        """
        self.set(packet)
        self.show()


    def set_simplified(self, simplify):
        """Builds Header accordingly.

        True - Header sends 16 bit universe value (OK but incorrect)
        False - Headers sends Universe - Net and Subnet values as protocol
        It is recommended that you set these values with .set_net() and set_physical
        """
        # avoid remaking header if there are no changes
        if simplify == self.is_simplified:
            return
        self.is_simplified = simplify
        self.make_artdmx_header()


    def see_header(self):
        """Show header values."""
        print(self.packet_header)


    def see_buffer(self):
        """Show buffer values."""
        print(self.buffer)


    def blackout(self):
        """Sends 0's all across."""
        self.clear()
        self.show()


    def flash_all(self, delay=None):
        """Sends 255's all across."""
        self.set([255] * self.packet_size)
        self.show()
        # Blackout after delay
        if delay:
            sleep(delay)
            self.blackout()


if __name__ == '__main__':
    print("===================================")
    print("Namespace run")
    TARGET_IP = '127.0.0.1'         # typically in 2.x or 10.x range
    UNIVERSE_TO_SEND = 15           # see docs
    PACKET_SIZE = 20                # it is not necessary to send whole universe

    a = StupidArtnet(TARGET_IP, UNIVERSE_TO_SEND, PACKET_SIZE, artsync=True)
    a.set_simplified(False)
    a.set_net(129)
    a.set_subnet(16)

    # Look at the object state
    print(a)

    a.set_single_value(13, 255)
    a.set_single_value(14, 100)
    a.set_single_value(15, 200)

    print("Sending values")
    a.show()
    a.see_buffer()
    a.flash_all()
    a.see_buffer()
    a.show()

    print("Values sent")

    # Cleanup when you are done
    del a



import unittest
from stupidArtnet.ArtnetUtils import shift_this, put_in_range, make_address_mask


class Test(unittest.TestCase):
    """Test class for Artnet Utilities."""

    def test_shift(self):
        """Test shift_this utility."""
        # send a 1
        self.assertEqual(shift_this(1), (0, 1))

        # send a 17
        self.assertEqual(shift_this(17), (0, 17))

        # send a 128
        self.assertEqual(shift_this(128), (0, 128))

        # send a 512
        self.assertEqual(shift_this(512), (2, 0))

        # send a 765
        self.assertEqual(shift_this(765), (2, 253))

        # send a 1024
        self.assertEqual(shift_this(1024), (4, 0))

        return True

    def test_range(self):
        """Test put_in_range utility."""
        # send a 1 from 0, 512, even
        self.assertEqual(put_in_range(1, 0, 512), (2))

        # send a 17 from 0, 18, even
        self.assertEqual(put_in_range(17, 0, 18), (18))

        # send a 128 from 0, 127
        self.assertEqual(put_in_range(128, 0, 127, False), (127))

        # send a 512 from 0, 512
        self.assertEqual(put_in_range(512, 0, 512), (512))

        # send a 1024 from 0, 512
        self.assertEqual(put_in_range(1024, 0, 512), (512))

        return True

    def test_mask(self):
        """Test mask and assert simplified mode."""
        self.assertEqual(make_address_mask(8), b'\x08\x00')
        self.assertEqual(make_address_mask(8, 0, 0, False), b'\x08\x00')

        self.assertEqual(make_address_mask(15), b'\x0f\x00')
        self.assertEqual(make_address_mask(15, 0, 0, False),  b'\x0f\x00')

        self.assertEqual(make_address_mask(16), b'\x10\x00')
        self.assertEqual(make_address_mask(0, 1, 0, False),  b'\x10\x00')

        self.assertEqual(make_address_mask(17), b'\x11\x00')
        self.assertEqual(make_address_mask(1, 1, 0, False),  b'\x11\x00')

        self.assertEqual(make_address_mask(18), b'\x12\x00')
        self.assertEqual(make_address_mask(2, 1, 0, False),  b'\x12\x00')
        self.assertEqual(2 + 1 * 16, 18)

        self.assertEqual(make_address_mask(99), b'c\x00')
        self.assertEqual(make_address_mask(3, 6, 0, False),  b'c\x00')
        self.assertEqual(3 + 6 * 16, 99)

        self.assertEqual(make_address_mask(255), b'\xff\x00')
        self.assertEqual(make_address_mask(15, 15, 0, False),  b'\xff\x00')
        self.assertEqual(15 + 15 * 16, 255)

        self.assertEqual(make_address_mask(256), b'\x00\x01')
        self.assertEqual(make_address_mask(0, 0, 1, False), b'\x00\x01')

        self.assertEqual(make_address_mask(257), b'\x01\x01')
        self.assertEqual(make_address_mask(1, 0, 1, False), b'\x01\x01')

        # with nets, it becomes difficult
        # to use the straight universe number
        self.assertEqual(make_address_mask(25736), b'\x88d')
        self.assertEqual(make_address_mask(8, 8, 100, False), b'\x88d')

        # Test clamp min
        self.assertEqual(make_address_mask(0), b'\x00\x00')
        self.assertEqual(make_address_mask(-15), b'\x00\x00')

        # Test clamp max
        self.assertEqual(make_address_mask(32767), b'\xff\x7f')
        self.assertEqual(make_address_mask(15, 15, 256, False), b'\xff\x7f')
        self.assertEqual(make_address_mask(999999), b'\xff\x7f')
        self.assertEqual(make_address_mask(99, 99, 300, False), b'\xff\x7f')

        return True


if __name__ == '__main__':
    unittest.main()



"""Facilitates library imports."""
from stupidArtnet.StupidArtnetServer import StupidArtnetServer
from stupidArtnet.ArtnetUtils import shift_this, put_in_range, make_address_mask
from .StupidArtnet import StupidArtnet



"""Provides common functions byte objects."""


def shift_this(number, high_first=True):
    """Utility method: extracts MSB and LSB from number.

    Args:
    number - number to shift
    high_first - MSB or LSB first (true / false)

    Returns:
    (high, low) - tuple with shifted values

    """
    low = (number & 0xFF)
    high = ((number >> 8) & 0xFF)
    if high_first:
        return((high, low))
    return((low, high))


def clamp(number, min_val, max_val):
    """Utility method: sets number in defined range.

    Args:
    number - number to use
    range_min - lowest possible number
    range_max - highest possible number

    Returns:
    number - number in correct range
    """
    return max(min_val, min(number, max_val))


def set_even(number):
    """Utility method: ensures number is even by adding.

    Args:
    number - number to make even

    Returns:
    number - even number
    """
    if number % 2 != 0:
        number += 1
    return number


def put_in_range(number, range_min, range_max, make_even=True):
    """Utility method: sets number in defined range.
    DEPRECATED: this will be removed from the library

    Args:
    number - number to use
    range_min - lowest possible number
    range_max - highest possible number
    make_even - should number be made even

    Returns:
    number - number in correct range

    """
    number = clamp(number, range_min, range_max)
    if make_even:
        number = set_even(number)
    return number


def make_address_mask(universe, sub=0, net=0, is_simplified=True):
    """Returns the address bytes for a given universe, subnet and net.

    Args:
    universe - Universe to listen
    sub - Subnet to listen
    net - Net to listen
    is_simplified - Whether to use nets and subnet or universe only,
    see User Guide page 5 (Universe Addressing)

    Returns:
    bytes - byte mask for given address

    """
    address_mask = bytearray()

    if is_simplified:
        # Ensure data is in right range
        universe = clamp(universe, 0, 32767)

        # Make mask
        msb, lsb = shift_this(universe)  # convert to MSB / LSB
        address_mask.append(lsb)
        address_mask.append(msb)
    else:
        # Ensure data is in right range
        universe = clamp(universe, 0, 15)
        sub = clamp(sub, 0, 15)
        net = clamp(net, 0, 127)

        # Make mask
        address_mask.append(sub << 4 | universe)
        address_mask.append(net & 0xFF)

    return address_mask



import socket
import unittest

from stupidArtnet import StupidArtnet


class Test(unittest.TestCase):
    """Test class for Artnet client."""

    # Art-Net stuff
    header_size = 18

    def setUp(self):
        """Creates UDP Server and Art-Net Client."""
        # Create dummy UDP Server
        self.sock = socket.socket(
            family=socket.AF_INET, type=socket.SOCK_DGRAM)
        self.sock.bind(('localhost', 6454))
        self.sock.settimeout(2000)

        # Instanciate stupidArtnet
        self.stupid = StupidArtnet(packet_size=24)

        # define a packet to send
        data = list(range(25))

        # send packet
        self.stupid.send(data)

        # confirm result
        self.received = self.sock.recv(512)

    def tearDown(self):
        """Destroy Objects."""
        # destroy UDP Server
        self.sock.close()

        # destroy artnet instance
        del self.stupid

    def test_header(self):
        """Assert Art-Net header."""
        # Art-Net header
        self.assertTrue(self.received.startswith(b'Art-Net'))

    def test_zero(self):
        """Check first data value, should be 0."""
        self.assertEqual(self.received[self.header_size], 0)

    def test_twelve(self):
        """Check twelfth data value, should be 12."""
        self.assertEqual(self.received[self.header_size] + 12, 12)

    def test_twentyfour(self):
        """Check twenty-fourth data value, should be 24."""
        self.assertEqual(self.received[self.header_size] + 24, 24)


if __name__ == '__main__':
    unittest.main()



from setuptools import setup

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setup(name='stupidArtnet',
      author='cpvalente',
      version='1.6.0',
      license='MIT',
      description='(Very) Simple implementation of the Art-Net protocol',
      long_description=long_description,
      long_description_content_type='text/markdown',
      url='https://github.com/cpvalente/stupidArtnet',
      project_urls={
          'Bug Tracker': 'https://github.com/cpvalente/stupidArtnet/issues',
      },
      classifiers=[
          'Programming Language :: Python :: 3',
          'License :: OSI Approved :: MIT License',
          'Operating System :: OS Independent',
      ],
      keywords=['LIGHTING', 'DMX', 'LIGHTING CONTROL'],
      packages=['stupidArtnet'],
      package_data={'stupidArtnet': ['examples/*']},
      python_requires=">=3.6")
